/**
 *
 */
package com.gic.demo.single.web.security;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.security.oauth2.client.EnableOAuth2Sso;
import org.springframework.boot.autoconfigure.web.ServerProperties;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.security.access.expression.method.DefaultMethodSecurityExpressionHandler;
import org.springframework.security.access.expression.method.MethodSecurityExpressionHandler;
import org.springframework.security.config.annotation.method.configuration.EnableGlobalMethodSecurity;
import org.springframework.security.config.annotation.method.configuration.GlobalMethodSecurityConfiguration;
import org.springframework.security.config.annotation.web.builders.HttpSecurity;
import org.springframework.security.config.annotation.web.configuration.WebSecurityConfigurerAdapter;
import org.springframework.security.web.authentication.logout.SecurityContextLogoutHandler;
import org.springframework.session.FindByIndexNameSessionRepository;
import org.springframework.session.data.redis.RedisOperationsSessionRepository;
import org.springframework.session.security.SpringSessionBackedSessionRegistry;
import org.springframework.web.context.request.RequestContextListener;
import org.springframework.web.cors.CorsUtils;

import com.gic.authcenter.security.core.authc.AuthcenterCookieClearingLogoutHandler;
import com.gic.authcenter.security.core.authc.AuthcenterLoginUrlAuthenticationEntryPoint;
import com.gic.authcenter.security.core.authc.AuthcenterLogoutSuccessHandler;
import com.gic.authcenter.security.core.authc.AuthcenterRedisSessionClearingLogoutHandler;
import com.gic.authcenter.security.core.authz.AuthcenterAccessDeniedHandler;
import com.gic.authcenter.security.core.authz.AuthcenterPermissionEvaluator;
import com.gic.demo.single.web.security.handler.CustomAuthenticationSuccessHandler;

/**
 *
 * @author leeon
 *
 */
@Configuration
@EnableOAuth2Sso
public class SecurityConfig extends WebSecurityConfigurerAdapter {

	@Autowired
	private ServerProperties serverProperties;

	@Autowired
	private RedisOperationsSessionRepository redisOperationsSessionRepository;

	@Bean
	@SuppressWarnings({ "unchecked", "rawtypes" })
	public SpringSessionBackedSessionRegistry sessionRegistry() {
		return new SpringSessionBackedSessionRegistry(((FindByIndexNameSessionRepository) redisOperationsSessionRepository));
	}

	@Bean
	public RequestContextListener requestContextListener() {
		return new RequestContextListener();
	}

	@Bean
	public CustomAuthenticationSuccessHandler customAuthenticationSuccessHandler() {
		return new CustomAuthenticationSuccessHandler();
	}

	@Bean
	public AuthcenterAccessDeniedHandler authcenterAccessDeniedHandler() {
		return new AuthcenterAccessDeniedHandler();
	}

	@Override
	public void configure(HttpSecurity http) throws Exception {
		// @formatter:off
		http
			.authorizeRequests()
			.antMatchers("/","/login")
			.permitAll()
			.requestMatchers(CorsUtils::isPreFlightRequest)
	        .permitAll()
			.anyRequest()
			.authenticated()
			.and()
			.formLogin()
	        .successHandler(customAuthenticationSuccessHandler())
	        .and()
	        .logout()
	        .addLogoutHandler(new AuthcenterCookieClearingLogoutHandler(
	        		serverProperties.getServlet().getSession().getCookie().getName()))
	        .addLogoutHandler(new AuthcenterRedisSessionClearingLogoutHandler(redisOperationsSessionRepository))
	        .addLogoutHandler(new SecurityContextLogoutHandler())
	        .logoutSuccessHandler(new AuthcenterLogoutSuccessHandler())
	        .and()
			.exceptionHandling()
	        .accessDeniedHandler(authcenterAccessDeniedHandler())
			.authenticationEntryPoint(new AuthcenterLoginUrlAuthenticationEntryPoint("/login"))
			.and()
			.sessionManagement()
	        .maximumSessions(1)
	        .sessionRegistry(sessionRegistry())
	        .and()
			.and().csrf().disable()
		;
		// @formatter:on
	}

	@Configuration
	@EnableGlobalMethodSecurity(prePostEnabled = true)
	public static class MethodSecurityConfig extends GlobalMethodSecurityConfiguration {

		@Bean
		public AuthcenterPermissionEvaluator authcenterPermissionEvaluator() {
			return new AuthcenterPermissionEvaluator();
		}

		@Override
		protected MethodSecurityExpressionHandler createExpressionHandler() {
			DefaultMethodSecurityExpressionHandler expressionHandler = new DefaultMethodSecurityExpressionHandler();
			expressionHandler.setPermissionEvaluator(authcenterPermissionEvaluator());
			return expressionHandler;
		}
	}

}
